' BLOCK.BAS

' A simple blocking HTTP client example
' This does exactly the same thing as ASYNC.BAS but uses blocking sockets.
' You may notice this is not as responsive as ASYNC.BAS - the program appears
' to freeze while it is waiting for data, but that's what you get for using
' blocking sockets.

' Include all functions defined in "wsockqb.bi"
REM $INCLUDE: 'wsockqb.bi'

CLS

' Call WSInit() to set up WSOCKQB. Remember if it returns -4 we have to run
' WSLADER.EXE and poll it until it returns 0.
ReturnValue% = WSInit
IF ReturnValue% = -4 THEN SHELL "WSLOADER.EXE"
t0! = TIMER
DO UNTIL ReturnValue% <> -4 OR TIMER - t0! > 5 ' Wait maximum 5 seconds
  ReturnValue% = WSInit
LOOP

PRINT "Initialising WSOCKXS...";
IF ReturnValue% <> 0 THEN
  ' Non-zero means error
  PRINT "ERROR:"; ReturnValue%
  END
ELSE
  ' Zero means success
  PRINT "done"
END IF

' Startup winsock. wsadataStruc is defined in "wsockqb.bi". On return
' wsadata's fields should be filled with stuff. (eg. wsadata.szDescription
' should contain a description of the winsock DLL)
DIM SHARED wsadata AS wsadataStruc
PRINT "Starting up winsock..."; WSAStartup(&H202, wsadata)

' Input a HTTP server name from user. The program checks to see if a user
' entered an IP address. If the user did, the program simply uses that IP
' address. If it was not an IP address, the program uses "ResolveName" to
' resolve the input.
INPUT "Enter HTTP server name:"; hn$
IF inetaddr(hn$) <> INADDRNONE THEN
  IPAddr$ = inetaddr(hn$)
ELSE
  ' Uses ResolveName SUB to resolve hostname. hn$ is the name the user
  ' entered earlier. All the other variables are named a$, b$ etc. because we
  ' don't need them.
  ResolveName hn$, a$, b$, c%, d%, IPAddr$, 0, e&
  PRINT "IP address: "; inetntoa(IPAddr$)
END IF

' Normally we would enable message handling here but since we are using
' blocking sockets we don't need it. Just to be sure lets disable it.
DisableMessage

' Create a simple TCP stream socket. This is the socket that HTTP uses. It is
' the basic TCP/IP socket.
NewSocket& = socket(AFINET, SOCKSTREAM, IPPROTOTCP)
PRINT "New socket: "; HEX$(NewSocket&)

' Connect the socket. Because we are using blocking routines no AsyncSelect()
' code is neccassary.
PRINT "Connecting...";

' Define connect address buffer
DIM SHARED ConnectBuffer AS sockaddr  ' Define our buffer
ConnectBuffer.sinfamily = 2           ' This usually always 2 (AFINET)
ConnectBuffer.sinport = htons(80)     ' Port number is stored in network byte
                                      ' order, so we have to use htons to
                                      ' convert it. 80 is the standard HTTP
                                      ' port.
ConnectBuffer.sinaddr = IPAddr$       ' IP address from previously.
ConnectBuffer.sinzero = STRING$(8, 0) ' This is always 8 nulls (CHR$(0)).

' Make the call
ConnectReturn& = connect(NewSocket&, ConnectBuffer, LEN(ConnectBuffer))

' connect() will return 0 if it was successful, -1 if an error happened. A
' specific error code may be returned by using WSAGetLastError().
IF ConnectReturn& = -1 THEN
  PRINT " error:"; WSAGetLastError
  GOTO closesock
END IF
PRINT "done"

' Send a very basic HTTP command down the connected socket. send() will
' return the number of bytes sent, or -1 if there is an error. Like
' connect(), a specific error code may be retrieved by using WSAGetLastError,
' but we don't use it here (which is bad programming, but oh well).
mybuf$ = "GET /index.html" + CHR$(13) + CHR$(10)
PRINT "BYTES SENT:"; send(NewSocket&, mybuf$, LEN(mybuf$), 0)

' Define recieve buffer
' Note: Recieve buffer MUST be a string
DIM SHARED MyRecv AS STRING * 1024

PRINT "Waiting for data..."

' Simple recieve loop. recv() returns the number of bytes recieved. This loop
' simply calls recv() until 0 bytes are returned (indicating the remote end
' closed socket gracefully).

' What does this program do with the data it recieves? Nothing. It only shows
' how to make a simple blocking client program that communicates with a HTTP
' server.
DO
  BytesRecv& = recv(NewSocket&, MyRecv, LEN(MyRecv), 0)
  PRINT "BYTES RECIEVED:"; BytesRecv&
LOOP UNTIL BytesRecv& = 0 OR BytesRecv& = -1

' If recv returned -1 then an error happened
IF BytesRecv& = -1 THEN PRINT "RECV ERROR:"; WSAGetLastError

closesock:

' Close the socket. You must close all open sockets at the end of your
' program to make sure everything has cleaned up properly.
PRINT "Closing socket..."; closesocket(NewSocket&)

' Cleanup winsock. One WSACleanup call is required for each WSAStartup()
' call.
PRINT "Cleaning up winsock..."; WSACleanup

' Close WSLOADER.EXE and unload WSOCKXS.VXD
SelfDestruct

