' PINGQB.BAS

' This example uses raw sockets to "ping" other IP addresses. Note that it
' seems under Windows NT/2000/XP this will NOT WORK unless you are logged in
' as administrator.

' This also supports a COMMAND$ interface so it's possible to compile this
' and use it just like PING.EXE (but this doesn't have as many options).
' If you are in the QB 4.5 IDE you will have to modify COMMAND$ (it's under
' the "Run" menu).

DECLARE FUNCTION CalcChecksum% (InOff AS INTEGER, InSeg AS INTEGER, length AS LONG)

' Include WSOCKXS definitions
REM $INCLUDE: 'wsockqb.bi'

' ICMP header type - 32 bytes long (you can make it more but you have to
' change other stuff)
TYPE ICMPhdr
  aType AS STRING * 1
  Code AS STRING * 1
  Checksum AS INTEGER
  ID AS INTEGER
  seqno AS INTEGER
  TimeStamp AS LONG
  DATA1 AS STRING * 20
END TYPE

DIM SHARED Packet AS ICMPhdr
DIM SHARED PacketID AS INTEGER
DIM SHARED ip AS STRING * 4
DIM SHARED AddressStructure AS sockaddr
DIM SHARED DB AS STRING * 1024
DIM SHARED TTL AS LONG

DIM SHARED Timeout AS LONG
Timeout = 2000  ' 2000 ms timeout (default)

TotalTimeTaken& = 0
TotalBytesSent& = 0
TotalBytesRecieved& = 0
TotalPacketsLost& = 0
TotalPacketsRecieved& = 0

TimesToPing% = 4 ' Do 4 pings (default)

' Parse command line
IF COMMAND$ = "" THEN GOTO showhelp
IF INSTR(COMMAND$, "/") THEN
  where% = INSTR(COMMAND$, "/")
  IF INSTR(COMMAND$, "/H") THEN GOTO showhelp
  IF INSTR(COMMAND$, "/T ") THEN
    where% = INSTR(COMMAND$, "/T ") + 3
    Timeout = VAL(MID$(COMMAND$, where%))
    IF Timeout = 0 THEN GOTO IncorrectSyntax
  END IF
  IF INSTR(COMMAND$, "/R ") THEN
    where% = INSTR(COMMAND$, "/R ") + 3
    TimesToPing% = VAL(MID$(COMMAND$, where%))
    IF TimesToPing% = 0 THEN GOTO IncorrectSyntax
  END IF
END IF

' Initialise WSOCKXS using WSInit()
ReturnValue% = WSInit
' If WSOCKXS.VXD is not loaded, run WSLOADER.EXE
IF ReturnValue% = -4 THEN SHELL "WSLOADER.EXE"
t0! = TIMER
DO UNTIL ReturnValue% <> -4 OR TIMER - t0! > 5 ' Wait maximum 5 seconds
  ReturnValue% = WSInit
LOOP

' Check for errors
IF ReturnValue% <> 0 THEN
  PRINT "Winsock could not be initialised:"; ReturnValue%
  END
END IF

' Startup winsock
DIM SHARED wsadata AS wsadataStruc
ret& = WSAStartup(&H202, wsadata)
IF ret& <> 0 THEN
  PRINT "ERROR: WSAStartup() returned:"; ret&
  GOTO doselfdestruct
END IF

' Enable message handling
DIM SHARED Message AS MessageStruc
EnableMessage

soff% = 0   ' soff% is the "string offset"
hn$ = ""    ' hn$ is the hostname
' Parse command line for hostname
DO
  soff% = soff% + 1
  IF MID$(COMMAND$, soff%, 1) = " " THEN EXIT DO
  IF MID$(COMMAND$, soff%, 1) = "/" THEN EXIT DO
  IF MID$(COMMAND$, soff%, 1) = "-" THEN EXIT DO
  hn$ = hn$ + MID$(COMMAND$, soff%, 1)
LOOP UNTIL soff% >= LEN(COMMAND$)

ip = inetaddr(hn$)
IF ip = INADDRNONE THEN
 
  ' hn$ is a hostname, so use ResolveName to resolve it into an IP address
  ResolveName hn$, a$, b$, c%, d%, ip, 0, Errors&
 
  ' Check for errors
  IF Errors& >= 11001 AND Errors& <= 11004 THEN
    PRINT "ERROR: Host " + hn$ + " does not exist"
    GOTO docleanup
  ELSEIF Errors& <> 0 THEN
    PRINT "ERROR: Resolve error:"; Errors&
    GOTO docleanup
  END IF

END IF

' Create a raw socket
NewSocket& = socket(AFINET, SOCKRAW, IPPROTOICMP)
IF NewSocket& = -1 THEN
  PRINT "ERROR: Socket could not be created:"; WSAGetLastError
  GOTO docleanup
END IF

' Set socket into asynchronous mode
Async& = WSAAsyncSelect(NewSocket&, GetHWnd, &H401, FDREAD)
IF Async& <> 0 THEN
  PRINT "ERROR: Socket could not be set into asynchronous mode"; WSAGetLastError
  GOTO CloseSock
END IF

PRINT
PRINT "Pinging " + inetntoa(ip) + " with"; LEN(Packet); "bytes of data"; TimesToPing%; "times:"

' Start pinging...
FOR numPings% = 1 TO TimesToPing%

  ' Get "random" packet ID
  RANDOMIZE TIMER
  PacketID = INT(RND * 1024) + 1

  ' Set up address structure
  AddressStructure.sinfamily = AFINET
  AddressStructure.sinport = 0
  AddressStructure.sinaddr = ip
  AddressStructure.sinzero = STRING$(8, 0)

  ' Send a ping packet to the IP address
  Packet.aType = CHR$(8)         ' Echo request
  Packet.Code = CHR$(0)          ' Umm.. I forget what this is
  Packet.Checksum = 0            ' 0 checksum for num
  Packet.ID = PacketID           ' Our "random" packet ID
  Packet.seqno = 0               ' First packet in sequence
  Packet.DATA1 = STRING$(24, 0)  ' Some useless data

  ' Calculate checksum
  check% = CalcChecksum(VARPTR(Packet), VARSEG(Packet), LEN(Packet))
  Packet.Checksum = check%

  ' Send the actual packet
  ' Note the use of PTR()
  ret& = sendto(NewSocket&, PTR(Packet), LEN(Packet), 0, AddressStructure, LEN(AddressStructure))
 
  ' Check for errors
  IF ret& = -1 THEN
    PRINT "ERROR: Packet could not be sent:"; WSAGetLastError
    GOTO CloseSock
  ELSE
    BytesSent = ret&
    StopTimer  ' Stop and reset timer
    ResetTimer
    StartTimer ' Restart timer
    TotalBytesSent& = TotalBytesSent& + BytesSent
  END IF

RecvAgain:

  ' Set recieve address structure for "all addresses"
  AddressStructure.sinfamily = 0
  AddressStructure.sinport = 0
  AddressStructure.sinaddr = STRING$(4, 0)
  AddressStructure.sinzero = STRING$(8, 0)

  ' Wait for reply
  DO
    TimeTaken& = GetTimer
    IF TimeTaken& >= Timeout THEN GOTO RecvTimeout
    IF INKEY$ = CHR$(27) THEN
      PRINT "ERROR: User aborted ping"
      GOTO CloseSock
    END IF
  LOOP UNTIL GetMessage(Message)

  ' Recieve the actual data
  ret& = recvfrom(NewSocket&, DB, LEN(DB), 0, AddressStructure, LEN(AddressStructure))

  ' Check for errors
  IF ret& = -1 THEN
    PRINT "ERROR: Packet could not be recieved:"; WSAGetLastError
    GOTO CloseSock
  ELSE
   
    ' Note that ret& = Bytes recieved
    ' See IP header format for more information on where the offsets of
    ' things are.
    IF CVI(MID$(DB, 25, 2)) <> PacketID THEN GOTO RecvAgain
    TotalBytesRecieved& = TotalBytesRecieved& + ret&
    TotalTimeTaken& = TotalTimeTaken& + TimeTaken&
   
    ' Print some statistics of the ping
    PRINT "  Reply from " + inetntoa(MID$(DB, 13, 4)) + ",";
    PRINT ret&; "bytes, time:";
    PRINT TimeTaken&; "ms, TTL:"; ASC(MID$(DB, 9, 1))
    TotalPacketsRecieved& = TotalPacketsRecieved& + 1

  END IF

  GOTO IsNotRecvTimeout

RecvTimeout:
  PRINT "  Ping timed out (>" + LTRIM$(STR$(Timeout)) + " ms)"
  TotalPacketsLost& = TotalPacketsLost& + 1

IsNotRecvTimeout:

NEXT numPings%

' Print overall ping statistics
PRINT
PRINT "Ping statistics:"
PRINT "  Total bytes sent:"; TotalBytesSent&; "bytes"
PRINT "  Total bytes recieved:"; TotalBytesRecieved&; "bytes"
IF TotalPacketsRecieved& <> 0 THEN
  PRINT "  Average round-trip:"; INT(TotalTimeTaken& / TotalPacketsRecieved&); "ms"
ELSE PRINT "  Average round-trip: 0 ms"
END IF
PRINT "  Packet loss: " + LTRIM$(STR$(INT(TotalPacketsLost& / numPings% * 100))) + "%"

CloseSock:

' Close the socket
junk& = closesocket(NewSocket&)

docleanup:

' Disable message handling
DisableMessage

' Cleanup winsock
junk& = WSACleanup

doselfdestruct:

' Unload WSOCKXS modules
SelfDestruct

END

IncorrectSyntax:
PRINT "Incorrect syntax"

showhelp:
PRINT
PRINT "PINGQB.EXE - A PINGer for QB 4.5"
PRINT "USAGE: PINGQB.EXE hostname [/h] [/t] [/r]"
PRINT
PRINT "/h - display help"
PRINT "/t - set timeout (in ms)"
PRINT "/r - set number of pings"
PRINT "eg. PINGQB 127.0.0.1 /t 100 /r 8"
PRINT "DEFAULT values are a 2000 ms timeout and 4 pings"

END

FUNCTION CalcChecksum% (InOff AS INTEGER, InSeg AS INTEGER, length AS LONG)

  ' Calculates IP checksum. I don't think it works. Qbasic doesn't do 16-bit
  ' unsigned adds (it does signed) and "overflow" cannot be utilised by
  ' Qbasic adds.

  DIM TempResult AS INTEGER
  bufferoffset% = 0
  ck& = 0
  DO
    FOR X% = 0 TO 1
      DEF SEG = VARSEG(InSeg)
      temp% = PEEK(InOff + bufferoffset% + X%)
      DEF SEG = VARSEG(TempResult)
      POKE VARPTR(TempResult) + X%, temp%
    NEXT X%
    d% = TempResult
    ck& = ck& + d%
    IF ck& >= 65536 THEN ck& = 65536 - ck&
    bufferoffset% = bufferoffset% + 2
    IF bufferoffset% >= length THEN EXIT DO
  LOOP

  FOR X% = 0 TO 1
    DEF SEG = VARSEG(ck&)
    temp% = PEEK(VARPTR(ck&) + X%)
    DEF SEG = VARSEG(TempResult)
    POKE VARPTR(TempResult) + X%, temp%
  NEXT X%

  CalcChecksum% = NOT (TempResult)

  DEF SEG

END FUNCTION

