DECLARE SUB QBTIMERSTART ()
DECLARE SUB QBTIMERSTOP ()
DECLARE SUB QBTIMERRESET ()
DECLARE FUNCTION QBTIMER! ()
' WSTIMER.BAS

' This simply demonstrates the timer functions. Qbasic's TIMER function only
' has a resolution of 18.2 Hz. The timer functions included with WSOCKXS have
' a resolution of 1000 Hz. Such a timer is best used for measuring ping
' times.

' Note that QBTIMERSTART, QBTIMERSTOP, QBTIMER and QBTIMERRESET are included
' to make TIMER into a "stopwatch" function like StartTimer, StopTimer and
' ResetTimer.

' Include WSOCKXS definitions
REM $INCLUDE: 'wsockqb.bi'

DIM SHARED QBTIMERSTOPPED AS INTEGER
DIM SHARED QBTIMEROFFSET AS SINGLE
DIM SHARED QBTIMERVALUE AS SINGLE
CLS

ReturnValue% = WSInit
' If WSOCKXS.VXD is not loaded, run WSLOADER.EXE
IF ReturnValue% = -4 THEN SHELL "WSLOADER.EXE"
t0! = TIMER
DO UNTIL ReturnValue% <> -4 OR TIMER - t0! > 1  ' Wait maximum one second
  ReturnValue% = WSInit
LOOP

' Check if there was an error in WSInit
IF ReturnValue% <> 0 THEN
  PRINT "ERROR: WSInit() returned:"; ReturnValue%
  END
END IF

' No message handling needed
DisableMessage

' Startup winsock
DIM SHARED wsadata AS wsadataStruc
ret& = WSAStartup(&H202, wsadata)
IF ret& <> 0 THEN
  PRINT "ERROR: WSAStartup() returned:"; ret&
  GOTO exitprogram
END IF

PRINT "WSTIMER.BAS - A simple timer test"
PRINT
PRINT "Press " + CHR$(34) + "a" + CHR$(34) + " to start the timers"
PRINT "Press " + CHR$(34) + "s" + CHR$(34) + " to stop the timers"
PRINT "Press " + CHR$(34) + "d" + CHR$(34) + " to reset the timers"
PRINT "Press ESC to exit"
PRINT
PRINT "Qbasic TIMER:"
PRINT "WSOCKXS GetTimer():"

' Stop and reset both timers
QBTIMERSTOP
QBTIMERRESET
StopTimer
ResetTimer

DO
  key$ = INKEY$
  LOCATE 8, 21: PRINT INT(QBTIMER * 1000)
  LOCATE 9, 21: PRINT GetTimer
  SELECT CASE key$
  CASE "a"
    StartTimer
    QBTIMERSTART
  CASE "s"
    StopTimer
    QBTIMERSTOP
  CASE "d"
    ResetTimer
    QBTIMERRESET
    ' Clear the timer values on screen. Sometimes if you reset the timers
    ' some "residue" numbers can be left on the screen.
    LOCATE 8, 21: PRINT SPACE$(20)
    LOCATE 9, 21: PRINT SPACE$(20)
  END SELECT
LOOP UNTIL key$ = CHR$(27)

LOCATE 11, 1: PRINT "Notice how GetTimer() updates so much faster than TIMER?"
PRINT "That is because TIMER updates only 18.2 times per second whereas GetTimer()"
PRINT "updates 1000 times a second. But then again, GetTimer() will not work in"
PRINT "pure dos (with no windows) but TIMER does. The reason for that is GetTimer()"
PRINT "uses the windows timers."

exitprogram:
ret& = WSACleanup ' Throw away WSACleanup()'s return value (we don't need it)
DisableMessage
SelfDestruct

FUNCTION QBTIMER!

  ' Equivalent to GetTimer() except using Qbasic's TIMER

  IF QBTIMERSTOPPED = 1 THEN
    QBTIMER! = QBTIMERVALUE
  ELSE
    QBTIMER! = TIMER - QBTIMEROFFSET + QBTIMERVALUE
  END IF

END FUNCTION

SUB QBTIMERRESET

  ' Equivalent to ResetTimer except using Qbasic's TIMER

  QBTIMERVALUE = 0
  QBTIMEROFFSET = TIMER

END SUB

SUB QBTIMERSTART

  ' Equivalent to StartTimer except using Qbasic's TIMER

  IF QBTIMERSTOPPED = 1 THEN
    QBTIMEROFFSET = TIMER
    QBTIMERSTOPPED = 0
  END IF

END SUB

SUB QBTIMERSTOP

  ' Equivalent to StopTimer except using Qbasic's TIMER

  IF QBTIMERSTOPPED = 0 THEN
    QBTIMERVALUE = QBTIMER
    QBTIMERSTOPPED = 1
  END IF

END SUB

